package jp.co.sds_corp.morphogenesis;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class TuringPattern
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	PointMap pointMap;
	List<Rule> listRule = new ArrayList<Rule>();

	public TuringPattern() {
		this(100, 100);
	}

	public TuringPattern(int x, int y) {
		pointMap = new PointMap(x, y);
	}

	public void init() throws Exception {
		loadRules();

		int all = 0;
		int mark = 0;
		for (int i = 0; i < pointMap.getX(); i++) {
			for (int j = 0; j < pointMap.getY(); j++) {
				int value = (int )(Math.random() * 2.0);
				if (value != 0) {
					mark++;
				}
				all++;
				pointMap.set(i, j, value);
/*
				if (j == 2) {
					logger.trace("(" + i + "," + j + ")=" + value);
				}
*/
			}
		}
		logger.debug(this, "all=" + all + " mark=" + mark);
	}

	protected void loadRules() throws Exception {
		URL url = getClass().getClassLoader().getResource( "morphogenesis.xml" );
		DOMConfiguration doc = new DOMConfiguration(url);
		List<IConfiguration> list = doc.listChild("rule");
		for (IConfiguration config : list) {
			String cls = config.getAttributes().get("class");
			Rule rule = (Rule )Class.forName(cls).newInstance();
			if (rule instanceof IConfigurable) {
				((IConfigurable)rule).init(config);
			}
			listRule.add(rule);
		}
	}

	public void start() {
		for (Rule rule: listRule) {
			for (int i = 0; i < pointMap.getX(); i++) {
				for (int j = 0; j < pointMap.getY(); j++) {
					rule.check(pointMap, i, j);
				}
			}
		}
		pointMap.doMark();
	}

	public PointMap getPointMap() {
		return pointMap;
	}

	public static void main(String[] args) {
		try {
			TuringPattern obj = new TuringPattern();
			obj.init();
			obj.start();
		} catch (Exception ex) {
			logger.error(TuringPattern.class, "error", ex);
			System.exit(9);
		}
	}

}
